#ifndef MCP_H_
#define MCP_H_

#include <avr/io.h>
#include "spi.h"

#define MCP_DDR				DDRA
#define MCP_PORT			PORTA
#define MCP_CS				PORTA7
#define MCP_IOCON_BANK		0x00

void mcp_init(uint8_t data);
void mcp_write_byte(uint8_t addr, uint8_t reg, uint8_t data);
void mcp_write_bit(uint8_t addr, uint8_t reg, uint8_t bit, uint8_t data);
uint8_t mcp_read_byte(uint8_t addr, uint8_t reg);
uint8_t mcp_read_bit(uint8_t addr, uint8_t reg, uint8_t bit);

#define MCP_OPCODE			0x40
#define MCP_WRITE			0x00
#define MCP_READ			0x01

#define MCP_PORTA			0x00
#if (MCP_IOCON_BANK == 0)
#define MCP_PORTB			0x01
#else
#define MCP_PORTB			0x10
#endif

//////////////////////////////////////////////////////////////////////////
// I/O DIRECTION REGISTER												//
// Controls the direction of the data I/O.								//
// When a bit is set, the corresponding pin becomes an input.			//
// When a bit is clear, the corresponding pin becomes an output.		//
//																		//
//	IO7-IO0:	These bits control the directon of data I/O				//
//				1 = Pin is configurated as an input						//
//				0 = Pin is configurated as output						//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_IODIR			0x00
#else
#define MCP_IODIR			0x00
#endif
#define MCP_IO0				0x01
#define MCP_IO1				0x02
#define MCP_IO2				0x04
#define MCP_IO3				0x08
#define MCP_IO4				0x0F
#define MCP_IO5				0x20
#define MCP_IO6				0x40
#define MCP_IO7				0x80

//////////////////////////////////////////////////////////////////////////
// INPUT POLARITY REGISTER												//
// This register allows the user to configure the polarity on the		//
// corresponding GPIO port bits.										//
// If a bit is set, the corresponding GPIO register bit will reflect	//
// the inverted value on the pin.										//
//																		//
//	IP7:IP0:		These bits control the polarity inversion of the	//
//					input pins											//
//					1 = GPIO register bit will reflect the opposite		//
//						logic state of the input pin.					//
//					0 = GPIO register bit will reflect the same logic	//
//						state of the input pin.							//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_IPOL			0x02
#else
#define MCP_IPOL			0x01
#endif
#define MCP_IPOL			0x02
#define MCP_IP0				0x01
#define MCP_IP1				0x02
#define MCP_IP2				0x04
#define MCP_IP3				0x08
#define MCP_IP4				0x0F
#define MCP_IP5				0x20
#define MCP_IP6				0x40
#define MCP_IP7				0x80

//////////////////////////////////////////////////////////////////////////
// INTERRUPT-ON-CHANGE CONTROL REGISTER									//
// The GPINTEN register controls the interrupt-onchange feature for		//
// each pin.															//
// If a bit is set, the corresponding pin is enabled for				//
// interrupt-on-change. The DEFVAL and INTCON registers must also be	//
// configured if any pins are enabled for interrupt-on-change.			//
//																		//
//	GPINT7:GPINT0:	General purpose I/O interrupt-on-change bits		//
//					1 = Enable GPIO input pin for interrupt-on-change	//
//						event.											//
//					0 = Disable GPIO input pin for interrupt-on-change	//
//						event.											//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_GPINTEN			0x04
#else
#define MCP_GPINTEN			0x02
#endif
#define MCP_GPINT0			0x01
#define MCP_GPINT1			0x02
#define MCP_GPINT2			0x04
#define MCP_GPINT3			0x08
#define MCP_GPINT4			0x0F
#define MCP_GPINT5			0x20
#define MCP_GPINT6			0x40
#define MCP_GPINT7			0x80

//////////////////////////////////////////////////////////////////////////
// DEFAULT COMPARE REGISTER FOR INTERRUPT-ON-CHANGE						//
// The default comparison value is configured in the DEFVAL register.	//
// If enabled (via GPINTEN and INTCON) to compare against the DEFVAL	//
// register, an opposite value on the associated pin will cause an		//
// interrupt to occur.													//
//																		//
//	DEF7:DEF0:		These bits set the compare value for pins			//
//					configured for interrupt-on-change from	defaults	//
//					If the associated pin level is the opposite from	//
//					the register bit, an interrupt occurs.				//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_DEFVAL			0x06
#else
#define MCP_DEFVAL			0x03
#endif
#define MCP_DEF0			0x01
#define MCP_DEF1			0x02
#define MCP_DEF2			0x04
#define MCP_DEF3			0x08
#define MCP_DEF4			0x0F
#define MCP_DEF5			0x20
#define MCP_DEF6			0x40
#define MCP_DEF7			0x80

//////////////////////////////////////////////////////////////////////////
// INTERRUPT CONTROL REGISTER											//
// The INTCON register controls how the associated pin value is			//
// compared for the interrupt-on-change feature.						//
// If a bit is set, the corresponding I/O pin is compared against the	//
// associated bit in the DEFVAL register. If a bit value is clear, the	//
// corresponding I/O pin is compared against the previous value.		//
//																		//
//	IOC7:IOC0:		These bits control how the associated pin value is	//
//					compared for interrupt-on-change					//
//					1 = Controls how the associated pin value is		//
//					compared for interrupt-on-change.					//
//					0 = Pin value is compared against the previous pin	//
//					value.												//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_INTCON			0x08
#else
#define MCP_INTCON			0x04
#endif
#define MCP_IOC0			0x01
#define MCP_IOC1			0x02
#define MCP_IOC2			0x04
#define MCP_IOC3			0x08
#define MCP_IOC4			0x0F
#define MCP_IOC5			0x20
#define MCP_IOC6			0x40
#define MCP_IOC7			0x80

//////////////////////////////////////////////////////////////////////////
// IOCON  I/O EXPANDER CONFIGURATION REGISTER							//
// The IOCON register contains several bits for configuring the device.	//
//																		//
//	BANK:			Controls how the registers are addressed			//
//					1 = The registers associated with each port are		//
//						separated into different banks					//
//					0 = The registers are in the same bank				//
//						(addresses are sequential)						//
//	MIRROR:			INT Pins Mirror bit									//
//					1 = The INT pins are internally connected			//
//					0 = The INT pins are not connected. INTA is			//
//						associated with PortA and INTB is associated	//
//						with PortB										//
//	SEQOP:			Sequential Operation mode bit.						//
//					1 = Sequential operation disabled, address pointer	//
//						does not increment.								//
//					0 = Sequential operation enabled, address pointer	//
//						 increments.									//
//	DISSLW:			Slew Rate control bit for SDA output.				//
//					1 = Slew rate disabled.								//
//					0 = Slew rate enabled.								//
//	HAEN:			Hardware Address Enable bit (MCP23S17 only).		//
//					Address pins are always enabled on MCP23017.		//
//					1 = Enables the MCP23S17 address pins.				//
//					0 = Disables the MCP23S17 address pins.				//
//	ODR:			This bit configures the INT pin as an open-drain	//
//					output.												//
//					1 = Open-drain output (overrides the INTPOL bit).	//
//					0 = Active driver output (INTPOL bit sets the		//
//					polarity).											//
//	INTPOL:			This bit sets the polarity of the INT output pin.	//
//					1 = Active-high.									//
//					0 = Active-low.										//
//	Unimplemented:	Read as 0.										//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_IOCON			0x0A
#else
#define MCP_IOCON			0x05
#endif
#define MCP_INTPOL			0x02
#define MCP_ODR				0x04
#define MCP_HAEN			0x08
#define MCP_DISSLW			0x0F
#define MCP_SEQOP			0x20
#define MCP_MIRROR			0x40
#define MCP_BANK			0x80

//////////////////////////////////////////////////////////////////////////
// PULL-UP RESISTOR CONFIGURATION REGISTER								//
// The GPPU register controls the pull-up resistors for the port pins.	//
// If a bit is set and the corresponding pin is configured as an input, //
// the corresponding port pin is internally pulled up with a 100 k?		//
// resistor.															//
//																		//
//	PU7:PU0:		These bits control the weak pull-up resistors on	//
//					each pin (when configured as an input)				//
//					1 = Pull-up enabled.								//
//					0 = Pull-up disabled.								//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_GPPU			0x0C
#else
#define MCP_GPPU			0x06
#endif
#define MCP_PU0				0x01
#define MCP_PU1				0x02
#define MCP_PU2				0x04
#define MCP_PU3				0x08
#define MCP_PU4				0x0F
#define MCP_PU5				0x20
#define MCP_PU6				0x40
#define MCP_PU7				0x80

//////////////////////////////////////////////////////////////////////////
// INTERRUPT FLAG REGISTER												//
// The INTF register reflects the interrupt condition on the port pins	//
// of any pin that is enabled for interrupts via the GPINTEN register.	//
// A set bit indicates that the associated pin caused the interrupt.	//
// This register is read-only. Writes to this register will be		//
// ignored.																//
//																		//
//	INT7:INT0:		These bits reflect the interrupt condition on the	//
//					port. Will reflect the change only if interrupts	//
//					are enabled (GPINTEN)								//
//					1 = Pin caused interrupt.							//
//					0 = Interrupt not pending.							//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_INTF			0x0E
#else
#define MCP_INTF			0x07
#endif
#define MCP_INT0			0x01
#define MCP_INT1			0x02
#define MCP_INT2			0x04
#define MCP_INT3			0x08
#define MCP_INT4			0x0F
#define MCP_INT5			0x20
#define MCP_INT6			0x40
#define MCP_INT7			0x80

//////////////////////////////////////////////////////////////////////////
// INTERRUPT CAPTURE REGISTER											//
// The INTCAP register captures the GPIO port value at the time the		//
// interrupt occurred. The register is read only and is updated only	//
// when an interrupt occurs. The register will remain unchanged until	//
// the interrupt is cleared via a read of INTCAP or GPIO.				//
//																		//
//	ICP7:ICP0:		These bits reflect the logic level on the port pins //
//					at the time of interrupt due to pin change			//
//					1 = Logic-high.										//
//					0 = Logic-low.										//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_INTCAP			0x10
#else
#define MCP_INTCAP			0x08
#endif
#define MCP_ICP0			0x01
#define MCP_ICP1			0x02
#define MCP_ICP2			0x04
#define MCP_ICP3			0x08
#define MCP_ICP4			0x0F
#define MCP_ICP5			0x20
#define MCP_ICP6			0x40
#define MCP_ICP7			0x80

//////////////////////////////////////////////////////////////////////////
// PORT REGISTER														//
// The GPIO register reflects the value on the port.					//
// Reading from this register reads the port. Writing to this register	//
// modifies the Output Latch (OLAT) register.							//
//																		//
//	GP7:GP0:		These bits reflect the logic level on the pins		//
//					1 = Logic-high.										//
//					0 = Logic-low.										//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_GPIO			0x12
#else
#define MCP_GPIO			0x09
#endif
#define MCP_GP0				0x01
#define MCP_GP1				0x02
#define MCP_GP2				0x04
#define MCP_GP3				0x08
#define MCP_GP4				0x0F
#define MCP_GP5				0x20
#define MCP_GP6				0x40
#define MCP_GP7				0x80

//////////////////////////////////////////////////////////////////////////
// OUTPUT LATCH REGISTER (OLAT)											//
// The OLAT register provides access to the output latches. A read from	//
// this register results in a read of the OLAT and not the port itself.	//
// A write to this register modifies the output latches that modifies	//
// the pins configured as outputs.										//
//																		//
//	OL7:OL0:		These bits reflect the logic level on the output	//
//					latch												//
//					1 = Logic-high.										//
//					0 = Logic-low.										//
//////////////////////////////////////////////////////////////////////////
#if	(MCP_IOCON_BANK == 0)
#define MCP_OLAT			0x14
#else
#define MCP_OLAT			0x0A
#endif
#define MCP_OL0				0x01
#define MCP_OL1				0x02
#define MCP_OL2				0x04
#define MCP_OL3				0x08
#define MCP_OL4				0x0F
#define MCP_OL5				0x20
#define MCP_OL6				0x40
#define MCP_OL7				0x80

#endif /* MCP_H_ */